# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/new_server_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_NewServerDialog(object):
    def setupUi(self, NewServerDialog):
        NewServerDialog.setObjectName("NewServerDialog")
        NewServerDialog.setWindowModality(QtCore.Qt.WindowModal)
        NewServerDialog.resize(426, 304)
        self.verticalLayout = QtWidgets.QVBoxLayout(NewServerDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(NewServerDialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiServerProtocolLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerProtocolLabel.setObjectName("uiServerProtocolLabel")
        self.gridLayout.addWidget(self.uiServerProtocolLabel, 0, 0, 1, 1)
        self.uiServerProtocolComboBox = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerProtocolComboBox.sizePolicy().hasHeightForWidth())
        self.uiServerProtocolComboBox.setSizePolicy(sizePolicy)
        self.uiServerProtocolComboBox.setObjectName("uiServerProtocolComboBox")
        self.uiServerProtocolComboBox.addItem("")
        self.uiServerProtocolComboBox.addItem("")
        self.gridLayout.addWidget(self.uiServerProtocolComboBox, 0, 1, 1, 2)
        self.uiServerHostLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerHostLabel.setObjectName("uiServerHostLabel")
        self.gridLayout.addWidget(self.uiServerHostLabel, 1, 0, 1, 1)
        self.uiServerHostLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiServerHostLineEdit.setObjectName("uiServerHostLineEdit")
        self.gridLayout.addWidget(self.uiServerHostLineEdit, 1, 1, 1, 2)
        self.uiServerPortLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerPortLabel.setObjectName("uiServerPortLabel")
        self.gridLayout.addWidget(self.uiServerPortLabel, 2, 0, 1, 1)
        self.uiServerPortSpinBox = QtWidgets.QSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiServerPortSpinBox.setSizePolicy(sizePolicy)
        self.uiServerPortSpinBox.setSuffix(" TCP")
        self.uiServerPortSpinBox.setMaximum(65535)
        self.uiServerPortSpinBox.setProperty("value", 3080)
        self.uiServerPortSpinBox.setObjectName("uiServerPortSpinBox")
        self.gridLayout.addWidget(self.uiServerPortSpinBox, 2, 1, 1, 2)
        self.uiEnableAuthenticationCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.uiEnableAuthenticationCheckBox.setObjectName("uiEnableAuthenticationCheckBox")
        self.gridLayout.addWidget(self.uiEnableAuthenticationCheckBox, 3, 0, 1, 3)
        self.uiServerUserLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerUserLabel.setObjectName("uiServerUserLabel")
        self.gridLayout.addWidget(self.uiServerUserLabel, 4, 0, 1, 2)
        self.uiServerUserLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiServerUserLineEdit.setEnabled(False)
        self.uiServerUserLineEdit.setToolTip("")
        self.uiServerUserLineEdit.setObjectName("uiServerUserLineEdit")
        self.gridLayout.addWidget(self.uiServerUserLineEdit, 4, 2, 1, 1)
        self.uiServerPasswordLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerPasswordLabel.setObjectName("uiServerPasswordLabel")
        self.gridLayout.addWidget(self.uiServerPasswordLabel, 5, 0, 1, 2)
        self.uiServerPasswordLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiServerPasswordLineEdit.setEnabled(False)
        self.uiServerPasswordLineEdit.setInputMethodHints(QtCore.Qt.ImhHiddenText|QtCore.Qt.ImhNoAutoUppercase|QtCore.Qt.ImhNoPredictiveText|QtCore.Qt.ImhSensitiveData)
        self.uiServerPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiServerPasswordLineEdit.setObjectName("uiServerPasswordLineEdit")
        self.gridLayout.addWidget(self.uiServerPasswordLineEdit, 5, 2, 1, 1)
        self.verticalLayout.addWidget(self.groupBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(NewServerDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        spacerItem = QtWidgets.QSpacerItem(20, 21, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(NewServerDialog)
        self.buttonBox.accepted.connect(NewServerDialog.accept)
        self.buttonBox.rejected.connect(NewServerDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(NewServerDialog)

    def retranslateUi(self, NewServerDialog):
        _translate = QtCore.QCoreApplication.translate
        NewServerDialog.setWindowTitle(_translate("NewServerDialog", "New server"))
        self.groupBox.setTitle(_translate("NewServerDialog", "Server settings"))
        self.uiServerProtocolLabel.setText(_translate("NewServerDialog", "Protocol:"))
        self.uiServerProtocolComboBox.setCurrentText(_translate("NewServerDialog", "HTTP"))
        self.uiServerProtocolComboBox.setItemText(0, _translate("NewServerDialog", "HTTP"))
        self.uiServerProtocolComboBox.setItemText(1, _translate("NewServerDialog", "HTTPS"))
        self.uiServerHostLabel.setText(_translate("NewServerDialog", "Host:"))
        self.uiServerHostLineEdit.setText(_translate("NewServerDialog", "192.168.56.101"))
        self.uiServerPortLabel.setText(_translate("NewServerDialog", "Port:"))
        self.uiEnableAuthenticationCheckBox.setText(_translate("NewServerDialog", "Enable authentication"))
        self.uiServerUserLabel.setText(_translate("NewServerDialog", "User:"))
        self.uiServerPasswordLabel.setText(_translate("NewServerDialog", "Password:"))

