# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/dynamips/ui/atm_bridge_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_atmBridgeConfigPageWidget(object):

    def setupUi(self, atmBridgeConfigPageWidget):
        atmBridgeConfigPageWidget.setObjectName("atmBridgeConfigPageWidget")
        atmBridgeConfigPageWidget.resize(432, 358)
        self.gridLayout_2 = QtWidgets.QGridLayout(atmBridgeConfigPageWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiMappingGroupBox = QtWidgets.QGroupBox(atmBridgeConfigPageWidget)
        self.uiMappingGroupBox.setObjectName("uiMappingGroupBox")
        self.vboxlayout = QtWidgets.QVBoxLayout(self.uiMappingGroupBox)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiMappingTreeWidget = QtWidgets.QTreeWidget(self.uiMappingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMappingTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiMappingTreeWidget.setSizePolicy(sizePolicy)
        self.uiMappingTreeWidget.setRootIsDecorated(False)
        self.uiMappingTreeWidget.setObjectName("uiMappingTreeWidget")
        self.vboxlayout.addWidget(self.uiMappingTreeWidget)
        self.gridLayout_2.addWidget(self.uiMappingGroupBox, 0, 2, 3, 1)
        self.uiEthernetGroupBox = QtWidgets.QGroupBox(atmBridgeConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetGroupBox.sizePolicy().hasHeightForWidth())
        self.uiEthernetGroupBox.setSizePolicy(sizePolicy)
        self.uiEthernetGroupBox.setObjectName("uiEthernetGroupBox")
        self.gridlayout = QtWidgets.QGridLayout(self.uiEthernetGroupBox)
        self.gridlayout.setObjectName("gridlayout")
        self.uiEthernetPortLabel = QtWidgets.QLabel(self.uiEthernetGroupBox)
        self.uiEthernetPortLabel.setObjectName("uiEthernetPortLabel")
        self.gridlayout.addWidget(self.uiEthernetPortLabel, 0, 0, 1, 1)
        self.uiEthernetPortSpinBox = QtWidgets.QSpinBox(self.uiEthernetGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEthernetPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiEthernetPortSpinBox.setSizePolicy(sizePolicy)
        self.uiEthernetPortSpinBox.setMinimum(0)
        self.uiEthernetPortSpinBox.setMaximum(65535)
        self.uiEthernetPortSpinBox.setProperty("value", 1)
        self.uiEthernetPortSpinBox.setObjectName("uiEthernetPortSpinBox")
        self.gridlayout.addWidget(self.uiEthernetPortSpinBox, 0, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiEthernetGroupBox, 1, 0, 1, 2)
        self.uiATMGroupBox = QtWidgets.QGroupBox(atmBridgeConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiATMGroupBox.sizePolicy().hasHeightForWidth())
        self.uiATMGroupBox.setSizePolicy(sizePolicy)
        self.uiATMGroupBox.setObjectName("uiATMGroupBox")
        self.gridlayout1 = QtWidgets.QGridLayout(self.uiATMGroupBox)
        self.gridlayout1.setObjectName("gridlayout1")
        self.uiATMPortLabel = QtWidgets.QLabel(self.uiATMGroupBox)
        self.uiATMPortLabel.setObjectName("uiATMPortLabel")
        self.gridlayout1.addWidget(self.uiATMPortLabel, 0, 0, 1, 1)
        self.uiATMPortSpinBox = QtWidgets.QSpinBox(self.uiATMGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiATMPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiATMPortSpinBox.setSizePolicy(sizePolicy)
        self.uiATMPortSpinBox.setMinimum(0)
        self.uiATMPortSpinBox.setMaximum(65535)
        self.uiATMPortSpinBox.setProperty("value", 10)
        self.uiATMPortSpinBox.setObjectName("uiATMPortSpinBox")
        self.gridlayout1.addWidget(self.uiATMPortSpinBox, 0, 1, 1, 1)
        self.uiATMVPILabel = QtWidgets.QLabel(self.uiATMGroupBox)
        self.uiATMVPILabel.setObjectName("uiATMVPILabel")
        self.gridlayout1.addWidget(self.uiATMVPILabel, 1, 0, 1, 1)
        self.uiATMVPISpinBox = QtWidgets.QSpinBox(self.uiATMGroupBox)
        self.uiATMVPISpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiATMVPISpinBox.sizePolicy().hasHeightForWidth())
        self.uiATMVPISpinBox.setSizePolicy(sizePolicy)
        self.uiATMVPISpinBox.setMinimum(0)
        self.uiATMVPISpinBox.setMaximum(65535)
        self.uiATMVPISpinBox.setSingleStep(1)
        self.uiATMVPISpinBox.setProperty("value", 0)
        self.uiATMVPISpinBox.setObjectName("uiATMVPISpinBox")
        self.gridlayout1.addWidget(self.uiATMVPISpinBox, 1, 1, 1, 1)
        self.uiATMVCILabel = QtWidgets.QLabel(self.uiATMGroupBox)
        self.uiATMVCILabel.setObjectName("uiATMVCILabel")
        self.gridlayout1.addWidget(self.uiATMVCILabel, 2, 0, 1, 1)
        self.uiATMVCISpinBox = QtWidgets.QSpinBox(self.uiATMGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiATMVCISpinBox.sizePolicy().hasHeightForWidth())
        self.uiATMVCISpinBox.setSizePolicy(sizePolicy)
        self.uiATMVCISpinBox.setMaximum(65535)
        self.uiATMVCISpinBox.setProperty("value", 100)
        self.uiATMVCISpinBox.setObjectName("uiATMVCISpinBox")
        self.gridlayout1.addWidget(self.uiATMVCISpinBox, 2, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiATMGroupBox, 2, 0, 1, 2)
        self.uiAddPushButton = QtWidgets.QPushButton(atmBridgeConfigPageWidget)
        self.uiAddPushButton.setObjectName("uiAddPushButton")
        self.gridLayout_2.addWidget(self.uiAddPushButton, 3, 0, 1, 1)
        self.uiDeletePushButton = QtWidgets.QPushButton(atmBridgeConfigPageWidget)
        self.uiDeletePushButton.setEnabled(False)
        self.uiDeletePushButton.setObjectName("uiDeletePushButton")
        self.gridLayout_2.addWidget(self.uiDeletePushButton, 3, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(371, 121, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 4, 0, 1, 3)
        self.uiGeneralGroupBox = QtWidgets.QGroupBox(atmBridgeConfigPageWidget)
        self.uiGeneralGroupBox.setObjectName("uiGeneralGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGeneralGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralGroupBox)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralGroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiGeneralGroupBox, 0, 0, 1, 2)

        self.retranslateUi(atmBridgeConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(atmBridgeConfigPageWidget)
        atmBridgeConfigPageWidget.setTabOrder(self.uiEthernetPortSpinBox, self.uiATMPortSpinBox)
        atmBridgeConfigPageWidget.setTabOrder(self.uiATMPortSpinBox, self.uiATMVPISpinBox)
        atmBridgeConfigPageWidget.setTabOrder(self.uiATMVPISpinBox, self.uiATMVCISpinBox)
        atmBridgeConfigPageWidget.setTabOrder(self.uiATMVCISpinBox, self.uiAddPushButton)
        atmBridgeConfigPageWidget.setTabOrder(self.uiAddPushButton, self.uiDeletePushButton)

    def retranslateUi(self, atmBridgeConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        atmBridgeConfigPageWidget.setWindowTitle(_translate("atmBridgeConfigPageWidget", "ATM Bridge"))
        self.uiMappingGroupBox.setTitle(_translate("atmBridgeConfigPageWidget", "Mapping"))
        self.uiMappingTreeWidget.headerItem().setText(0, _translate("atmBridgeConfigPageWidget", "Ethernet Port"))
        self.uiMappingTreeWidget.headerItem().setText(1, _translate("atmBridgeConfigPageWidget", "Port:VPI:VCI"))
        self.uiEthernetGroupBox.setTitle(_translate("atmBridgeConfigPageWidget", "Ethernet side"))
        self.uiEthernetPortLabel.setText(_translate("atmBridgeConfigPageWidget", "Port:"))
        self.uiATMGroupBox.setTitle(_translate("atmBridgeConfigPageWidget", "ATM side"))
        self.uiATMPortLabel.setText(_translate("atmBridgeConfigPageWidget", "Port:"))
        self.uiATMVPILabel.setText(_translate("atmBridgeConfigPageWidget", "VPI:"))
        self.uiATMVCILabel.setText(_translate("atmBridgeConfigPageWidget", "VCI:"))
        self.uiAddPushButton.setText(_translate("atmBridgeConfigPageWidget", "&Add"))
        self.uiDeletePushButton.setText(_translate("atmBridgeConfigPageWidget", "&Delete"))
        self.uiGeneralGroupBox.setTitle(_translate("atmBridgeConfigPageWidget", "General"))
        self.uiNameLabel.setText(_translate("atmBridgeConfigPageWidget", "Name:"))
